function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { if (typeof Symbol === "undefined" || !(Symbol.iterator in Object(arr))) return; var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

import { getBreakpoints, getMediaqueries, getQueriesObjects } from './utils';
import { useEffect } from 'react';
import PropTypes from 'prop-types';
import React, { useMemo, useRef, useState } from 'react';
import ResponsiveContext from './ResponsiveContext';

var ResponsiveProvider = function ResponsiveProvider(_ref) {
  var initialMediaType = _ref.initialMediaType,
      defaultOrientation = _ref.defaultOrientation,
      children = _ref.children,
      breakpoints = _ref.breakpoints,
      breakpointsMax = _ref.breakpointsMax,
      mediaQueries = _ref.mediaQueries;

  var _useState = useState(false),
      _useState2 = _slicedToArray(_useState, 2),
      isCalculated = _useState2[0],
      setIsCalculated = _useState2[1];

  var _useState3 = useState(initialMediaType),
      _useState4 = _slicedToArray(_useState3, 2),
      currentBreakpoint = _useState4[0],
      setCurrentBreakpoint = _useState4[1];

  var _useState5 = useState(defaultOrientation),
      _useState6 = _slicedToArray(_useState5, 2),
      currentOrientation = _useState6[0],
      setCurrentOrientation = _useState6[1];

  var breakpointsWithInitialValue = _objectSpread({
    _initial: '0em'
  }, breakpoints);

  var breakpointNames = Object.keys(mediaQueries || breakpointsWithInitialValue);

  if (!mediaQueries) {
    mediaQueries = getMediaqueries(breakpointsWithInitialValue, breakpointsMax, breakpointNames);
  }

  var breakpointsRef = useRef(getBreakpoints(mediaQueries, breakpointNames));
  var queriesObjects = useMemo(function () {
    return getQueriesObjects(currentBreakpoint, breakpointsRef.current);
  }, [currentBreakpoint, breakpointsRef]); // Add event listener for breakpoints

  useEffect(function () {
    var breakpoints = breakpointsRef.current;
    var localMediaQueries = breakpoints.map(function (breakpoint) {
      return breakpoint.mediaQuery;
    });
    var mediaQueryListsAndListeners = localMediaQueries.map(function (mediaQuery, index) {
      var mediaQueryList = window.matchMedia(mediaQuery);

      if (mediaQueryList.matches) {
        setCurrentBreakpoint(breakpoints[index].mediaType);
        setIsCalculated(true);
      }

      var listener = function listener(event) {
        event.matches && setCurrentBreakpoint(breakpoints[index].mediaType);
      };

      mediaQueryList.addListener(listener);
      return [mediaQueryList, listener];
    });
    return function () {
      mediaQueryListsAndListeners.forEach(function (_ref2) {
        var _ref3 = _slicedToArray(_ref2, 2),
            mediaQueryList = _ref3[0],
            listener = _ref3[1];

        mediaQueryList.removeListener(listener);
      });
    };
  }, [breakpointsRef]); // Add orientation media query

  useEffect(function () {
    var setMatchedOrientation = function setMatchedOrientation(matches) {
      setCurrentOrientation(matches ? 'portrait' : 'landscape');
    };

    var orientationMediaQueryList = window.matchMedia('(orientation: portrait)');
    setMatchedOrientation(orientationMediaQueryList.matches);

    var listener = function listener(event) {
      setMatchedOrientation(event.matches);
    };

    orientationMediaQueryList.addListener(listener);
    return function () {
      orientationMediaQueryList.removeListener(listener);
    };
  }, []);
  var contextObject = useMemo(function () {
    return _objectSpread({
      mediaType: currentBreakpoint,
      orientation: currentOrientation,
      isCalculated: isCalculated
    }, queriesObjects);
  }, [currentBreakpoint, currentOrientation, isCalculated, queriesObjects]);
  useEffect(function () {
    if (!process || !process.env || process.env.NODE_ENV !== 'test') {
      /* eslint-disable no-console */
      console.group('%c @farfetch/react-context-responsive updated!', 'color:#c4aa84; font-weight:bold;');
      console.log('%c Current breakpoint:', 'font-weight:bold;', currentBreakpoint);
      console.log('%c Context:', 'font-weight:bold;', contextObject);
      console.groupEnd();
      /* eslint-enable no-console */
    }
  }, [contextObject, currentBreakpoint]);
  return /*#__PURE__*/React.createElement(ResponsiveContext.Provider, {
    value: contextObject
  }, children);
};

var breakpointsPropTypes = {
  xs: PropTypes.string,
  sm: PropTypes.string,
  md: PropTypes.string,
  lg: PropTypes.string,
  xl: PropTypes.string
};
ResponsiveProvider.defaultProps = {
  initialMediaType: 'xs',
  defaultOrientation: null
};
ResponsiveProvider.propTypes = {
  initialMediaType: PropTypes.oneOf(['_initial', 'xs', 'sm', 'md', 'lg', 'xl']),
  defaultOrientation: PropTypes.oneOf(['landscape', 'portrait']),
  children: PropTypes.oneOfType([PropTypes.arrayOf(PropTypes.node), PropTypes.node]).isRequired,
  breakpoints: PropTypes.exact(breakpointsPropTypes),
  breakpointsMax: PropTypes.exact(breakpointsPropTypes),
  mediaQueries: PropTypes.exact(_objectSpread(_objectSpread({}, breakpointsPropTypes), {}, {
    _initial: PropTypes.string
  }))
};
export default ResponsiveProvider;